<?php

namespace Drupal\zammad_webform_handler\Plugin\WebformHandler;

use Drupal\Core\Url;
use ZammadAPIClient\Client;
use ZammadAPIClient\ResourceType;
use Drupal\Core\Form\FormStateInterface;
use Drupal\webform\Plugin\WebformHandlerBase;
use Drupal\webform\WebformSubmissionInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Webform submission remote post handler.
 *
 * @WebformHandler(
 *   id = "zammad",
 *   label = @Translation("Zammad"),
 *   category = @Translation("External"),
 *   description = @Translation("Posts webform submissions to zammad."),
 *   cardinality = \Drupal\webform\Plugin\WebformHandlerInterface::CARDINALITY_UNLIMITED,
 *   results = \Drupal\webform\Plugin\WebformHandlerInterface::RESULTS_PROCESSED,
 *   submission = \Drupal\webform\Plugin\WebformHandlerInterface::SUBMISSION_OPTIONAL,
 *   tokens = TRUE,
 * )
 */
class ZammadWebformHandler extends WebformHandlerBase {

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The HTTP client to fetch the feed data with.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected $httpClient;

  /**
   * The token manager.
   *
   * @var \Drupal\webform\WebformTokenManagerInterface
   */
  protected $tokenManager;

  /**
   * The webform message manager.
   *
   * @var \Drupal\webform\WebformMessageManagerInterface
   */
  protected $messageManager;

  /**
   * The webform element plugin manager.
   *
   * @var \Drupal\webform\Plugin\WebformElementManagerInterface
   */
  protected $elementManager;

  /**
   * The current request.
   *
   * @var \Symfony\Component\HttpFoundation\Request
   */
  protected $request;

  /**
   * The request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * The DrupalKernel instance used in the test.
   *
   * @var \Drupal\Core\DrupalKernel
   */
  protected $kernel;

  /**
   * List of unsupported webform submission properties.
   *
   * The below properties will not being included in a remote post.
   *
   * @var array
   */
  protected $unsupportedProperties = [
    'metatag',
  ];

  /**
   * The zammad_webform_handler config.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $externalConfig;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->externalConfig = $container->get('config.factory')->get('zammad_webform_handler.settings');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'ticket_title' => '',
      'ticket_subject' => '',
      'ticket_body' => '',
      'ticket_group_id' => '',
      'ticket_customer_id' => '',
      'ticket_customer_mail' => '',
      'ticket_priority_id' => '',
      'ticket_state_id' => '',
      'ticket_type' => '',
      'ticket_internal' => '',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form['zammad_webform_handler_settings'] = [
      '#type' => 'details',
      '#open' => TRUE,
      '#title' => $this->t('Zammad Handler Settings'),
      '#description' => $this->t('Make sure the authentication method, the zammad url and the http token are set and make sure your zammad installation supports Zammad API Calls!'),
    ];
    $form['zammad_webform_handler_settings']['link'] = [
      '#type' => 'link',
      '#title' => $this->t('Go to generell Zammad Settings'),
      '#url' => Url::fromRoute('zammad_webform_handler.settings_form'),
    ];
    $form['zammad_webform_handler_settings']['required_settings'] = [
      '#type' => 'details',
      '#open' => TRUE,
      '#title' => $this->t('Required Settings'),
    ];
    $form['zammad_webform_handler_settings']['optional_settings'] = [
      '#type' => 'details',
      '#open' => FALSE,
      '#title' => $this->t('Optional Settings'),
    ];

    // REQUIRED SETTINGS:
    $form['zammad_webform_handler_settings']['required_settings']['ticket_title'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Zammad Ticket Title'),
      '#default_value' => $this->configuration['ticket_title'],
      '#required' => TRUE,
    ];
    $form['zammad_webform_handler_settings']['required_settings']['ticket_subject'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Zammad Ticket Subject'),
      '#default_value' => $this->configuration['ticket_subject'],
      '#required' => TRUE,
    ];
    $form['zammad_webform_handler_settings']['required_settings']['ticket_body'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Zammad Ticket Body'),
      '#default_value' => $this->configuration['ticket_body'],
      '#required' => TRUE,
    ];
    $form['zammad_webform_handler_settings']['required_settings']['ticket_group_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Zammad Ticket Group ID'),
      '#default_value' => $this->configuration['ticket_group_id'],
      '#required' => TRUE,
    ];
    $form['zammad_webform_handler_settings']['required_settings']['ticket_customer_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Zammad Ticket Customer ID'),
      '#description' => $this->t('If left empty, "Zammad Ticket Customer Mail" will be used instead.'),
      '#default_value' => $this->configuration['ticket_customer_id'],
    ];
    $form['zammad_webform_handler_settings']['required_settings']['ticket_customer_mail'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Zammad Ticket Customer Mail'),
      '#default_value' => $this->configuration['ticket_customer_mail'],
    ];

    // OPTIONAL SETTINGS:
    $form['zammad_webform_handler_settings']['optional_settings']['ticket_priority_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Zammad Ticket Priority ID'),
      '#default_value' => $this->configuration['ticket_priority_id'],
    ];
    $form['zammad_webform_handler_settings']['optional_settings']['ticket_state_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Zammad Ticket State ID'),
      '#default_value' => $this->configuration['ticket_state_id'],
    ];
    $form['zammad_webform_handler_settings']['optional_settings']['ticket_type'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Zammad Ticket Type'),
      '#default_value' => $this->configuration['ticket_type'],
    ];
    $form['zammad_webform_handler_settings']['optional_settings']['ticket_internal'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Whether or not it is an internal ticket.'),
      '#default_value' => $this->configuration['ticket_internal'],
    ];

    return $this->setSettingsParents($form);
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);
    $this->applyFormStateToConfiguration($form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state) {
    if (empty($this->externalConfig->get('authentication_method'))) {
      $form_state->setErrorByName('zammad_webform_handler_settings_link', $this->t('Please choose a authentication method on the settings page!'));
    }
    if (empty($this->externalConfig->get('zammad_url'))) {
      $form_state->setErrorByName('zammad_webform_handler_settings_link', $this->t('Please define a zammad url on the settings page!'));
    }
    if (empty($this->externalConfig->get('http_token_secret'))) {
      $form_state->setErrorByName('zammad_webform_handler_settings_link', $this->t('Please define a HTTP Token on the settings page!'));
    }
    // @todo if time
    // if (empty($form_state->getValue('ticket_customer_id')) && empty($form_state->getValue('ticket_customer_mail'))) {
    //   $form_state->setErrorByName('ticket_customer_id', $this->t('Either the Zammad Ticket Customer ID or the Zammad Ticket Customer Mail should be set!'));
    // }
    return parent::validateConfigurationForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function postSave(WebformSubmissionInterface $webform_submission, $update = TRUE): void {
    $this->sendZammadTicket();
  }

  /**
   * Sends a Zammad Ticket a zammad installation.
   */
  protected function sendZammadTicket(): void {
    if (empty($this->externalConfig->get('authentication_method')) || empty($this->externalConfig->get('zammad_url')) || empty($this->externalConfig->get('http_token_secret'))) {
      $this->getLogger('zammad_webform_handler')
        ->error('Error: The authentication_method, zammad_url or http_token_secret are empty.');
      return;
    }

    $client = new Client([
      'url'           => $this->externalConfig->get('zammad_url'),
      'http_token'      => $this->externalConfig->get('http_token_secret'),
      'timeout'       => $this->externalConfig->get('timeout'),
      'verify'       => $this->externalConfig->get('verify'),
    ]);

    $ticket = $client->resource(ResourceType::TICKET);

    $customerId = $this->configuration['ticket_customer_id'];
    if (empty($customerId)) {
      $customerMail = $this->configuration['ticket_customer_mail'];
      if (!empty($customerMail)) {
        $customerId = "guess:" . $customerMail;
      }
    }

    // Setup required ticket data:
    $ticket_data = [
      'group_id'    => $this->configuration['ticket_group_id'],
      'title'       => $this->configuration['ticket_title'],
      'customer_id' => $customerId,
      'article'     => [
        'subject' => $this->configuration['ticket_subject'],
        'body'    => $this->configuration['ticket_body'],
      ],
    ];

    // Add optional ticket data:
    if (!empty($type = $this->configuration['ticket_type'])) {
      $ticket_data['article']['type'] = $type;
    }
    if (!empty($internal = $this->configuration['ticket_internal'])) {
      $ticket_data['article']['ticket_internal'] = $internal;
    }
    if (!empty($priorityId = $this->configuration['ticket_priority_id'])) {
      $ticket_data['ticket_priority_id'] = $priorityId;
    }
    if (!empty($stateId = $this->configuration['ticket_state_id'])) {
      $ticket_data['state_id'] = $stateId;
    }
    // Set ticket values and send ticket:
    $ticket->setValues($ticket_data);
    $ticket->save();
    if ($ticket->hasError()) {
      $this->getLogger('zammad_webform_handler')
        ->error($ticket->getError());
    }
  }

}
